@@ -1,4 +1,4 @@
-## Copyright (C) 2009-2015 Philip Nienhuis
+## Copyright (C) 2009-2016 Philip Nienhuis
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -59,13 +59,20 @@
 ##
 ## Optional argument @var{options}, a structure, can be used to specify
 ## various write modes.
-## Currently the only option field is "formulas_as_text", which -if set
-## to 1 or TRUE- specifies that formula strings (i.e., text strings
-## starting with "=" and ending in a ")" ) should be entered as litteral
+## @table @asis
+## @item "formulas_as_text"
+## If set to 1 or TRUE formula strings ( i.e., text strings (assumed to
+## start with "=" and end in a ")" ) are to be written as litteral
 ## text strings rather than as spreadsheet formulas (the latter is the
 ## default).  As jOpenDocument doesn't support formula I/O at all yet,
 ## this option is ignored for the JOD interface.
 ##
+## @item 'convert_utf'
+## If set to 1 or TRUE, oct2ods converts one-byte characters outside the
+## range [32:127] to UTF-8 so that they are properly entered as UTF-8
+## encoded text in spreadsheets.  The default value is 0.
+## @end table
+##
 ## Data are added to the sheet, ignoring other data already present;
 ## existing data in the range to be used will be overwritten.
 ##
@@ -77,12 +84,12 @@
 ## Examples:
 ##
 ## @example
-##   [ods, status] = ods2oct (arr, ods, 'Newsheet1', 'AA31:GH165');
+##   [ods, status] = oct2ods (arr, ods, 'Newsheet1', 'AA31:GH165');
 ##   Write array arr into sheet Newsheet1 with upperleft cell at AA31
 ## @end example
 ##
 ## @example
-##   [ods, status] = ods2oct (@{'String'@}, ods, 'Oldsheet3', 'B15:B15');
+##   [ods, status] = oct2ods (@{'String'@}, ods, 'Oldsheet3', 'B15:B15');
 ##   Put a character string into cell B15 in sheet Oldsheet3
 ## @end example
 ##
@@ -135,38 +142,24 @@ function [ ods, rstatus ] = oct2ods (c_arr, ods, wsh=1, crange="", spsh_opts=[])
     error ("oct2ods: character string (range) expected for arg # 4\n");
   elseif (isempty (crange))
     crange = "";
-  else
-    ## FIXME the following stanza can be made into a separate func
+  elseif (! isempty (crange))
     ## Check for range name and convert it to range & optionally sheet
     ## 1. Check if it matches a range
-    mtch = cell2mat (regexp (crange, ...
-                     '(^[A-Za-z]+[0-9]+){1}(:[A-Za-z]+[0-9]+$)?', "tokens"));
-    if (! isempty (mtch) && ! strcmp ([mtch{:}], crange))
-      ## Apparently not a range. Try range names
-      if (! isfield (xls, "nmranges"))
-        xls.nmranges = getnmranges (xls);
-      endif
-      idx = strmatch (crange, xls.nmranges(:, 1));
-      if (isempty (idx))
-        error ("oct2xls: no range '%s' in workbook", crange);
-      else
-        ## Get range and -optionally- sheet it refers to
-        crange = xls.nmranges{idx, 3};
-        if (! isempty (xls.nmranges{idx, 2}))
-          wsh = xls.nmranges{idx, 2};
-        endif
-      endif
-    endif
+    [crange, wsh, ods] = chknmrange (ods, crange, wsh);
   endif
 
   ## Various options 
   if (isempty (spsh_opts))
     spsh_opts.formulas_as_text = 0;
+    spsh_opts.convert_utf = 0;
     ## other options to be implemented here
   elseif (isstruct (spsh_opts))
     if (! isfield (spsh_opts, "formulas_as_text"))
       spsh_opts.formulas_as_text = 0; 
     endif
+    if (! isfield (spsh_opts, "convert_utf"))
+      spsh_opts.convert_utf = ! ispc;
+    endif
     ## other options to be implemented here:
  
   else
@@ -177,6 +170,13 @@ function [ ods, rstatus ] = oct2ods (c_arr, ods, wsh=1, crange="", spsh_opts=[])
     printf ("oct2ods: warning: no output spreadsheet file pointer specified.\n");
   endif
 
+  ## Convert to UTF-8
+  ## FIXME: Replace with the following when unicode2native is available:
+  ##        obj = tidyxml (obj, @(str) unicode2native (str, "UTF-8"));
+  if (spsh_opts.convert_utf)
+    c_arr = tidyxml (c_arr, @unicode2utf8);
+  endif
+
   if (strcmp (ods.xtype, "OTK"))
     ## Write ods file tru Java & ODF toolkit.
     switch ods.odfvsn
@@ -197,6 +197,13 @@ function [ ods, rstatus ] = oct2ods (c_arr, ods, wsh=1, crange="", spsh_opts=[])
     [ ods, rstatus ] = __UNO_oct2spsh__ (c_arr, ods, wsh, crange, spsh_opts);
 
   elseif (strcmp (ods.xtype, "OCT"))
+    ## Replace illegal XML characters by XML escape sequences
+    idx = cellfun (@ischar, c_arr);
+    c_arr(idx) = strrep (c_arr(idx), "&", "&amp;" );
+    c_arr(idx) = strrep (c_arr(idx), "<", "&lt;"  );
+    c_arr(idx) = strrep (c_arr(idx), ">", "&gt;"  );
+    c_arr(idx) = strrep (c_arr(idx), "'", "&apos;");
+    c_arr(idx) = strrep (c_arr(idx), '"', "&quot" );
     ## Write ods or gnumeric file tru native Octave
     [ ods, rstatus ] = __OCT_oct2spsh__ (c_arr, ods, wsh, crange, spsh_opts);
 
