@@ -63,12 +63,21 @@
 ##
 ## Optional argument @var{options}, a structure, can be used to specify
 ## various write modes.
-## Currently the only option field is "formulas_as_text", which -if set
-## to 1 or TRUE- specifies that formula strings (i.e., text strings
-## starting with "=" and ending in a ")" ) should be entered as litteral
+## @table @asis
+## @item "formulas_as_text"
+## If set to 1 or TRUE formula strings ( i.e., text strings (assumed to
+## start with "=" and end in a ")" ) are to be written as litteral
 ## text strings rather than as spreadsheet formulas (the latter is the
 ## default).
 ##
+## @item 'convert_utf'
+## If set to 1 or TRUE, oct2xls converts one-byte characters outside the
+## range [32:127] to UTF-8 so that they are properly entered as UTF-8
+## encoded text in spreadsheets.  The default value is 0.
+## This setting has no effect for the COM interface as that does the
+## encoding automatically using libraries outside Octave.
+## @end table
+##
 ## Beware that -if invoked- Excel invocations may be left running silently
 ## in case of COM errors.  Invoke xlsclose with proper pointer struct to
 ## close them.
@@ -145,11 +154,15 @@ function [ xls, rstatus ] = oct2xls (obj, xls, wsh=1, crange="", spsh_opts=[])
   ## Various options 
   if (isempty (spsh_opts))
     spsh_opts.formulas_as_text = 0;
+    spsh_opts.convert_utf = 0;
     ## other options to be implemented here
   elseif (isstruct (spsh_opts))
     if (! isfield (spsh_opts, "formulas_as_text"))
       spsh_opts.formulas_as_text = 0; 
     endif
+    if (! isfield (spsh_opts, "convert_utf"))
+      spsh_opts.convert_utf = ! ispc;
+    endif
     ## other options to be implemented here
 
   else
@@ -159,6 +172,13 @@ function [ xls, rstatus ] = oct2xls (obj, xls, wsh=1, crange="", spsh_opts=[])
   if (nargout < 1)
     printf ("oct2xls: warning: no output spreadsheet file pointer specified.\n");
   endif
+
+  ## Convert to UTF-8
+  ## FIXME: Replace with the following when unicode2native is available:
+  ##        obj = tidyxml (obj, @(str) unicode2native (str, "UTF-8"));
+  if (! strcmp (xls.xtype, "COM") && (spsh_opts.convert_utf))
+    obj = tidyxml (obj, @unicode2utf8);
+  endif
   
   ## Select interface to be used
   if (strcmpi (xls.xtype, "COM"))
@@ -177,9 +197,6 @@ function [ xls, rstatus ] = oct2xls (obj, xls, wsh=1, crange="", spsh_opts=[])
     ## Invoke Java and UNO bridge (OpenOffice.org)
     [xls, rstatus] = __UNO_oct2spsh__ (obj, xls, wsh, crange, spsh_opts);
   elseif (strcmpi (xls.xtype, "OCT"))
-    ## Clean up obj from characters that'll be converted to double-byte
-    ## by Excel/LibreOffice (any chars < 32 or > 126)
-    obj = tidyxml (obj);
     ## Replace illegal XML characters by XML escape sequences
     idx = cellfun (@ischar, obj);
     obj(idx) = strrep (obj(idx), "&", "&amp;" );
