@@ -1,4 +1,4 @@
-## Copyright (C) 2009-2015 Philip Nienhuis
+## Copyright (C) 2009-2016 Philip Nienhuis
 ## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -105,7 +105,24 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
       error ("xlsopen.m: numerical or logical value expected for arg ## 2 (readwrite)\n")
   endif
 
+  if (ischar (filename))
+    [pth, fnam, ext] = fileparts (filename);
+    if (isempty (fnam))
+      error ("xlsopen.m: no filename or empty filename specified");
+    endif
+    if (xwrite && ! isempty (pth))
+      apth = make_absolute_filename (pth);
+      if (exist (apth) != 7)
+        error ("xlsopen.m: cannot write into non-existent directory:\n'%s'\n", ...
+               apth);
+      endif
+    endif
+  else
+    error ("xlsopen.m: filename expected for argument #1");
+  endif
+
   if (! isempty (reqinterface))
+    intfmsg = "requested";
     if (! (ischar (reqinterface) || iscell (reqinterface)))
       error ("Arg ## 3 (interface) not recognized - character value required\n"); 
     endif
@@ -159,6 +176,8 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
         return
       endif
     endif
+  else
+    intfmsg = "available";
   endif
 
   ## Check if Excel file exists. First check for (supported) file name suffix:
@@ -275,7 +294,7 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
   endif
 
   if ((! xlssupport) && xlsinterfaces.JXL && ftype == 1)
-    [ xls, xlssupport, lastintf ] = __JXL_spsh_open__ (xls, xwrite, filename, xlssupport, ftype);
+    [ xls, xlssupport, lastintf ] = __JXL_spsh_open__ (xls, xwrite, filename, xlssupport, ftype, xlsinterfaces);
   endif
 
   if ((! xlssupport) && xlsinterfaces.OXS && ftype == 1)
@@ -307,10 +326,11 @@ function [ xls ] = xlsopen (filename, xwrite=0, reqinterface=[])
       if (ftype != 6)
         ## This message is appended after message from getxlsinterfaces()
         printf ("None.\n");
-        warning ("xlsopen.m: no support for '%s' spreadsheet I/O\n", ext);
+        warning ("xlsopen.m: no'%s' spreadsheet I/O support with %s interfaces.\n", ...
+                 ext, intfmsg);
       endif
     else
-      ## No match between filte type & interface found
+      ## No match between file type & interface found
       warning ("xlsopen.m: file type not supported by %s %s %s %s %s %s\n", reqinterface{:});
     endif
     xls = [];
